@@ -1,19 +1,21 @@
 # -*- coding: utf-8 -*-
-import os
 
 from chaoslib.exceptions import FailedActivity
 from chaoslib.types import Configuration, Secrets
 from logzero import logger
 
 from chaosazure import init_client
-from chaosazure.machine.constants import RES_TYPE_VM, OS_LINUX, OS_WINDOWS
-from chaosazure.rgraph.resource_graph import fetch_resources
-
+from chaosazure.common import cleanse
+from chaosazure.common.compute import command
+from chaosazure.machine.constants import RES_TYPE_VM
+from chaosazure.common.resources.graph import fetch_resources
 
 __all__ = ["delete_machines", "stop_machines", "restart_machines",
            "start_machines", "stress_cpu", "fill_disk", "network_latency",
            "burn_io"]
 
+from chaosazure.vmss.records import Records
+
 
 def delete_machines(filter: str = None,
                     configuration: Configuration = None,
@@ -50,11 +52,15 @@ def delete_machines(filter: str = None,
 
     machines = __fetch_machines(filter, configuration, secrets)
     client = __compute_mgmt_client(secrets, configuration)
+    machine_records = Records()
     for m in machines:
         group = m['resourceGroup']
         name = m['name']
         logger.debug("Deleting machine: {}".format(name))
         client.virtual_machines.delete(group, name)
+        machine_records.add(cleanse.machine(m))
+
+    return machine_records.output_as_dict('resources')
 
 
 def stop_machines(filter: str = None,
@@ -89,11 +95,16 @@ def stop_machines(filter: str = None,
 
     machines = __fetch_machines(filter, configuration, secrets)
     client = __compute_mgmt_client(secrets, configuration)
+
+    machine_records = Records()
     for m in machines:
         group = m['resourceGroup']
         name = m['name']
         logger.debug("Stopping machine: {}".format(name))
         client.virtual_machines.power_off(group, name)
+        machine_records.add(cleanse.machine(m))
+
+    return machine_records.output_as_dict('resources')
 
 
 def restart_machines(filter: str = None,
@@ -128,11 +139,15 @@ def restart_machines(filter: str = None,
 
     machines = __fetch_machines(filter, configuration, secrets)
     client = __compute_mgmt_client(secrets, configuration)
+    machine_records = Records()
     for m in machines:
         group = m['resourceGroup']
         name = m['name']
         logger.debug("Restarting machine: {}".format(name))
         client.virtual_machines.restart(group, name)
+        machine_records.add(cleanse.machine(m))
+
+    return machine_records.output_as_dict('resources')
 
 
 def start_machines(filter: str = None,
@@ -169,7 +184,16 @@ def start_machines(filter: str = None,
     machines = __fetch_machines(filter, configuration, secrets)
     client = __compute_mgmt_client(secrets, configuration)
     stopped_machines = __fetch_all_stopped_machines(client, machines)
-    __start_stopped_machines(client, stopped_machines)
+
+    machine_records = Records()
+    for machine in stopped_machines:
+        logger.debug("Starting machine: {}".format(machine['name']))
+        client.virtual_machines.start(machine['resourceGroup'],
+                                      machine['name'])
+
+        machine_records.add(cleanse.machine(machine))
+
+    return machine_records.output_as_dict('resources')
 
 
 def stress_cpu(filter: str = None,
@@ -178,7 +202,7 @@ def stress_cpu(filter: str = None,
                configuration: Configuration = None,
                secrets: Secrets = None):
     """
-    Stress CPU up to 100% at random machines.
+    Stress CPU up to 100% at virtual machines.
 
     Parameters
     ----------
@@ -210,30 +234,17 @@ def stress_cpu(filter: str = None,
     Stress two machines at random from the group 'rg'
     """
 
-    logger.debug(
-        "Start stress_cpu: configuration='{}', filter='{}'".format(
-            configuration, filter))
+    msg = "Starting stress_cpu:" \
+          " configuration='{}', filter='{}', duration='{}', timeout='{}'" \
+        .format(configuration, filter, duration, timeout)
+    logger.debug(msg)
 
     machines = __fetch_machines(filter, configuration, secrets)
-    client = __compute_mgmt_client(secrets, configuration)
 
-    for m in machines:
-        name = m['name']
-        group = m['resourceGroup']
-        os_type = __get_os_type(m)
-        if os_type == OS_WINDOWS:
-            command_id = 'RunPowerShellScript'
-            script_name = "cpu_stress_test.ps1"
-        elif os_type == OS_LINUX:
-            command_id = 'RunShellScript'
-            script_name = "cpu_stress_test.sh"
-        else:
-            raise FailedActivity(
-                "Cannot run CPU stress test on OS: %s" % os_type)
-
-        with open(os.path.join(os.path.dirname(__file__),
-                               "../scripts", script_name)) as file:
-            script_content = file.read()
+    machine_records = Records()
+    for machine in machines:
+        command_id, script_content = command \
+            .prepare(machine, 'cpu_stress_test')
 
         parameters = {
             'command_id': command_id,
@@ -243,21 +254,21 @@ def stress_cpu(filter: str = None,
             ]
         }
 
-        logger.debug("Stressing CPU of machine: {}".format(name))
-        poller = client.virtual_machines.run_command(group, name, parameters)
-        result = poller.result(duration + timeout)  # Blocking till executed
-        if result:
-            logger.debug(result.value[0].message)  # stdout/stderr
-        else:
-            raise FailedActivity(
-                "stress_cpu operation did not finish on time. "
-                "You may consider increasing timeout setting.")
+        logger.debug("Stressing CPU of machine: '{}'".format(machine['name']))
+        _timeout = duration + timeout
+        command.run(
+            machine['resourceGroup'], machine, _timeout, parameters,
+            secrets, configuration)
+        machine_records.add(cleanse.machine(machine))
+
+    return machine_records.output_as_dict('resources')
 
 
 def fill_disk(filter: str = None,
               duration: int = 120,
               timeout: int = 60,
               size: int = 1000,
+              path: str = None,
               configuration: Configuration = None,
               secrets: Secrets = None):
     """
@@ -277,6 +288,9 @@ def fill_disk(filter: str = None,
         recommended to set this value to less than 30s. Defaults to 60 seconds.
     size : int
         Size of the file created on the disk. Defaults to 1GB.
+    path : str, optional
+        The absolute path to write the fill file into.
+        Defaults: C:/burn for Windows clients, /root/burn for Linux clients.
 
 
     Examples
@@ -296,51 +310,36 @@ def fill_disk(filter: str = None,
     Fill two machines at random from the group 'rg'
     """
 
-    logger.debug(
-        "Start fill_disk: configuration='{}', filter='{}'".format(
-            configuration, filter))
+    msg = "Starting fill_disk: configuration='{}', filter='{}'," \
+          " duration='{}', size='{}', path='{}', timeout='{}'" \
+        .format(configuration, filter, duration, size, path, timeout)
+    logger.debug(msg)
 
     machines = __fetch_machines(filter, configuration, secrets)
-    client = __compute_mgmt_client(secrets, configuration)
 
-    for m in machines:
-        name = m['name']
-        group = m['resourceGroup']
-        os_type = __get_os_type(m)
-        if os_type == OS_WINDOWS:
-            command_id = 'RunPowerShellScript'
-            script_name = "fill_disk.ps1"
-        elif os_type == OS_LINUX:
-            command_id = 'RunShellScript'
-            script_name = "fill_disk.sh"
-        else:
-            raise FailedActivity(
-                "Cannot run disk filling test on OS: %s" % os_type)
+    machine_records = Records()
+    for machine in machines:
+        command_id, script_content = command.prepare(machine, 'fill_disk')
+        fill_path = command.prepare_path(machine, path)
 
-        with open(os.path.join(os.path.dirname(__file__),
-                               "../scripts", script_name)) as file:
-            script_content = file.read()
-
-        logger.debug("Script content: {}".format(script_content))
         parameters = {
             'command_id': command_id,
             'script': [script_content],
             'parameters': [
                 {'name': "duration", 'value': duration},
-                {'name': "size", 'value': size}
+                {'name': "size", 'value': size},
+                {'name': "path", 'value': fill_path}
             ]
         }
 
-        logger.debug("Filling disk of machine: {}".format(name))
-        poller = client.virtual_machines.run_command(group, name, parameters)
-        result = poller.result(duration + timeout)  # Blocking till executed
-        logger.debug("Execution result: {}".format(poller))
-        if result:
-            logger.debug(result.value[0].message)  # stdout/stderr
-        else:
-            raise FailedActivity(
-                "fill_disk operation did not finish on time. "
-                "You may consider increasing timeout setting.")
+        logger.debug("Filling disk of machine: {}".format(machine['name']))
+        _timeout = duration + timeout
+        command.run(
+            machine['resourceGroup'], machine, _timeout, parameters,
+            secrets, configuration)
+        machine_records.add(cleanse.machine(machine))
+
+    return machine_records.output_as_dict('resources')
 
 
 def network_latency(filter: str = None,
@@ -394,22 +393,11 @@ def network_latency(filter: str = None,
             configuration, filter))
 
     machines = __fetch_machines(filter, configuration, secrets)
-    client = __compute_mgmt_client(secrets, configuration)
 
-    for m in machines:
-        name = m['name']
-        group = m['resourceGroup']
-        os_type = __get_os_type(m)
-        if os_type == OS_LINUX:
-            command_id = 'RunShellScript'
-            script_name = "network_latency.sh"
-        else:
-            raise FailedActivity(
-                "Cannot run network latency test on OS: %s" % os_type)
-
-        with open(os.path.join(os.path.dirname(__file__),
-                               "../scripts", script_name)) as file:
-            script_content = file.read()
+    machine_records = Records()
+    for machine in machines:
+        command_id, script_content = command \
+            .prepare(machine, 'network_latency')
 
         logger.debug("Script content: {}".format(script_content))
         parameters = {
@@ -422,16 +410,15 @@ def network_latency(filter: str = None,
             ]
         }
 
-        logger.debug("Increasing the latency of machine: {}".format(name))
-        poller = client.virtual_machines.run_command(group, name, parameters)
-        result = poller.result(duration + timeout)  # Blocking till executed
-        logger.debug("Execution result: {}".format(poller))
-        if result:
-            logger.debug(result.value[0].message)  # stdout/stderr
-        else:
-            raise FailedActivity(
-                "network_latency operation did not finish on time. "
-                "You may consider increasing timeout setting.")
+        logger.debug("Increasing the latency of machine: {}"
+                     .format(machine['name']))
+        _timeout = duration + timeout
+        command.run(
+            machine['resourceGroup'], machine, _timeout, parameters,
+            secrets, configuration)
+        machine_records.add(cleanse.machine(machine))
+
+    return machine_records.output_as_dict('resources')
 
 
 def burn_io(filter: str = None,
@@ -474,58 +461,37 @@ def burn_io(filter: str = None,
     the group 'rg'
     """
 
-    logger.debug(
-        "Start burn_io: configuration='{}', filter='{}'".format(
-            configuration, filter))
+    msg = "Starting burn_io: configuration='{}', filter='{}', duration='{}'," \
+          " timeout='{}'".format(configuration, filter, duration, timeout)
+    logger.debug(msg)
 
     machines = __fetch_machines(filter, configuration, secrets)
-    client = __compute_mgmt_client(secrets, configuration)
 
-    for m in machines:
-        name = m['name']
-        group = m['resourceGroup']
-        os_type = __get_os_type(m)
-        if os_type == OS_LINUX:
-            command_id = 'RunShellScript'
-            script_name = "burn_io.sh"
-        else:
-            raise FailedActivity(
-                "Cannot run burn_io test on OS: %s" % os_type)
+    machine_records = Records()
+    for machine in machines:
+        command_id, script_content = command.prepare(machine, 'burn_io')
 
-        with open(os.path.join(os.path.dirname(__file__),
-                               "../scripts", script_name)) as file:
-            script_content = file.read()
-
-        logger.debug("Script content: {}".format(script_content))
         parameters = {
             'command_id': command_id,
             'script': [script_content],
             'parameters': [
-                {'name': "duration", 'value': duration},
+                {'name': "duration", 'value': duration}
             ]
         }
 
-        logger.debug("Increasing the I/O operations per "
-                     "second of machine: {}".format(name))
-        poller = client.virtual_machines.run_command(group, name, parameters)
-        result = poller.result(duration + timeout)  # Blocking till executed
-        logger.debug("Execution result: {}".format(poller))
-        if result:
-            logger.debug(result.value[0].message)  # stdout/stderr
-        else:
-            raise FailedActivity(
-                "burn_io operation did not finish on time. "
-                "You may consider increasing timeout setting.")
+        logger.debug("Burning IO of machine: '{}'".format(machine['name']))
+        _timeout = duration + timeout
+        command.run(
+            machine['resourceGroup'], machine, _timeout, parameters,
+            secrets, configuration)
+        machine_records.add(cleanse.machine(machine))
+
+    return machine_records.output_as_dict('resources')
 
 
 ###############################################################################
 # Private helper functions
 ###############################################################################
-def __start_stopped_machines(client, stopped_machines):
-    for machine in stopped_machines:
-        logger.debug("Starting machine: {}".format(machine['name']))
-        client.virtual_machines.start(machine['resourceGroup'],
-                                      machine['name'])
 
 
 def __fetch_all_stopped_machines(client, machines) -> []:
@@ -554,14 +520,5 @@ def __fetch_machines(filter, configuration, secrets) -> []:
     return machines
 
 
-def __get_os_type(machine):
-    os_type = machine['properties']['storageProfile']['osDisk']['osType'] \
-        .lower()
-    if os_type not in (OS_LINUX, OS_WINDOWS):
-        raise FailedActivity("Unknown OS Type: %s" % os_type)
-
-    return os_type
-
-
 def __compute_mgmt_client(secrets, configuration):
     return init_client(secrets, configuration)
